#! /usr/bin/env python
"""
Read data from a NetCDF file into a RasterModelGrid.
"""

try:
    import netCDF4 as nc4
except ImportError:
    import warnings
    warnings.warn('Unable to import netCDF4.', ImportWarning)

from scipy.io import netcdf as nc

import os
import types
import re

import numpy as np

from landlab import RasterModelGrid
from landlab.io.netcdf.errors import NotRasterGridError
from landlab.io.netcdf._constants import (_AXIS_DIMENSION_NAMES,
                                          _AXIS_COORDINATE_NAMES,
                                          _COORDINATE_NAMES)


def _length_of_axis_dimension(root, axis_name):
    try:
        return len(root.dimensions[axis_name])
    except TypeError:
        return root.dimensions[axis_name]


def _read_netcdf_grid_shape(root):
    shape = []
    for axis_name in _AXIS_DIMENSION_NAMES:
        try:
            shape.append(_length_of_axis_dimension(root, axis_name))
        except KeyError:
            pass
    return shape


def _read_netcdf_coordinate_values(root):
    values = []
    for coordinate_name in _AXIS_COORDINATE_NAMES:
        try:
            values.append(root.variables[coordinate_name][:])
        except KeyError:
            pass
    return values


def _read_netcdf_coordinate_units(root):
    units = []
    for coordinate_name in _AXIS_COORDINATE_NAMES:
        try:
            units.append(root.variables[coordinate_name].units)
        except KeyError:
            pass
    return units


def _read_netcdf_structured_grid(root):
    shape = _read_netcdf_grid_shape(root)
    coordinates = _read_netcdf_coordinate_values(root)
    units = _read_netcdf_coordinate_units(root)

    for coordinate in coordinates:
        coordinate.shape = shape

    return coordinates


def _read_netcdf_structured_data(root):
    fields = dict()
    for (name, var) in root.variables.items():
        if not name in _COORDINATE_NAMES:
            fields[name] = var[:].copy()
            fields[name].shape = (fields[name].size, )
    return fields


def _get_raster_spacing(coords):
    spacing = np.empty(len(coords), dtype=np.float64)

    for (axis, coord) in enumerate(coords):
        coord_spacing = np.diff(coord, axis=axis)
        try:
            assert(np.all(coord_spacing == coord_spacing.flat[0]))
        except AssertionError:
            raise NotRasterGridError()
        spacing[axis] = coord_spacing.flat[0]

    try:
        assert(np.all(spacing == spacing[0]))
    except AssertionError:
        raise NotRasterGridError()
    else:
        return spacing[0]


def read_netcdf(nc_file, reshape=False, just_grid=False):
    try:
        root = nc.netcdf_file(nc_file, 'r', version=2)
    except TypeError:
        root = nc4.Dataset(nc_file, 'r', format='NETCDF4')

    node_coords = _read_netcdf_structured_grid(root)

    assert(len(node_coords) == 2)

    spacing = _get_raster_spacing(node_coords)

    shape = node_coords[0].shape

    grid = RasterModelGrid(num_rows=shape[0], num_cols=shape[1],
                           dx=spacing)

    if not just_grid:
        fields = _read_netcdf_structured_data(root)
        for (name, values) in fields.items():
            grid.add_field('node', name, values)

    root.close()

    return grid



#! /usr/bin/env python
"""
The ModelDataFields class holds at set of ScalarDataFields that are separated
into groups. A typical use for this class would be to define the groups as
being locations on a grid where the values are defined. For instance, the
groups could be *node*, *cell*, *link*, and *face*.

Most of the method functions for ModelDataFields are the same as those for
the ScalarDataFields class but with the first argument being a string that
defines the group name.

Create a sets of data fields defined at *node* and *cell*. Each set can
have a differenct number of values.

>>> fields = ModelDataFields()
>>> fields.new_field_location('node', 12)
>>> fields.new_field_location('cell', 2)

Create some new value arrays for each of the data fields.

>>> fields.ones('node')
array([ 1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.])
>>> fields.zeros('cell')
array([ 0.,  0.])

Create new value arrays and add them to the data fields. Because the data
fields are in different groups (node and cell), they can have the same
name.

>>> fields.add_ones('node', 'planet_surface__elevation')
>>> fields.at_node['planet_surface__elevation']
array([ 1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.,  1.])

>>> fields.add_ones('cell', 'planet_surface__elevation')
>>> fields.at_cell['planet_surface__elevation']
array([ 1.,  1.])
"""

import types
import inspect

from landlab.field import ScalarDataFields


class ModelDataFields(object):
    def __init__(self, **kwds):
        #print 'ModelDataFields.__init__'
        self._groups = dict()
        super(ModelDataFields, self).__init__(**kwds)

    @property
    def groups(self):
        return set(self._groups.keys())

    def has_group(self, group):
        return group in self._groups

    def new_field_location(self, group, size):
        if self.has_group(group):
            raise ValueError('ModelDataFields already contains %s' % group)
        else:
            self._groups[group] = ScalarDataFields(size)
            setattr(self, 'at_' + group, self[group])

    def field_values(self, group, field):
        return self[group][field]

    def field_units(self, group, field):
        return self[group].units[field]

    def __getitem__(self, group):
        return self._groups[group]


def get_method_from_class(clazz, method_name):
    method = getattr(clazz, method_name)
    if inspect.ismethod(method):
        return method
    else:
        raise TypeError('%s is not a bound method' % method_name)


def is_regular_method_name(name):
    return not (name.startswith('__') and name.endswith('__'))


def regular_method_names(clazz):
    methods = set()
    for (name, _) in inspect.getmembers(clazz, inspect.ismethod):
        if is_regular_method_name(name):
            methods.add(name)
    return methods


def prepend_arg_list_with_dict_value(destination_class, method):
    def func_with_dict_value_first(self, group, *args, **kwds):
        return method(self[group], *args, **kwds)
    func_with_dict_value_first.__doc__ = method.__doc__
    func_with_dict_value_first.__name__ = method.__name__
    return types.MethodType(func_with_dict_value_first, None,
                            destination_class)


def add_methods_to_grouped_fields_class():
    for name in (regular_method_names(ScalarDataFields) -
                 regular_method_names(ModelDataFields)):
        method = get_method_from_class(ScalarDataFields, name)
        new_method = prepend_arg_list_with_dict_value(ModelDataFields,
                                                      method)
        setattr(ModelDataFields, name, new_method)


add_methods_to_grouped_fields_class()



from landlab.components.craters.dig_craters import impactor
from landlab import ModelParameterDictionary

from landlab import RasterModelGrid
import numpy as np
import pylab
from scipy.optimize import curve_fit

#get the needed properties to build the grid:
input_file = './craters_params_init.txt'
inputs = ModelParameterDictionary(input_file)
nrows = inputs.read_int('nrows')
ncols = inputs.read_int('ncols')
dx = inputs.read_float('dx')
leftmost_elev = inputs.read_float('leftmost_elevation')
initial_slope = inputs.read_float('initial_slope')
nt = inputs.read_int('number_of_craters_per_loop')
loops = inputs.read_int('number_of_loops')

mg = RasterModelGrid(nrows, ncols, dx)
mg.set_looped_boundaries(True, True)
mg.create_node_array_zeros('planet_surface__elevation')

def fitFunc(t, a,b,c,d,e,f,g):
    return a*t**6. + b*t**5. +c*t**4. + d*t**3. + e*t**2. + f*t + g



#instantiate the component:
craters_component = impactor(mg, input_file)

params_from_first_try = np.array([ -2.10972667e+02,   3.23669793e+02,  -2.01070114e+02,
                                    7.40429578e+01,  -1.77124298e+01,   2.03540786e-01,
                                   -1.01168519e-01])
list_of_mass_bals = []
param_collection = np.empty_like(params_from_first_try)

slope_values = np.arange(0., 51.)/100.
angle_values = np.arange(0., 85.)
beta = []

print 'Beginning loop...'

repeats = 10
work_with = slope_values

for i in range(repeats):
    mass_balance = []
    for k in work_with:
        #print 'Slope is ', k
        #create the fields in the grid
        if work_with.size == slope_values.size:
            initial_slope = k
        elif work_with.size == angle_values.size:
            craters_component._angle_to_vertical = k*np.pi/180.
        z = mg.create_node_array_zeros() + leftmost_elev
        z += initial_slope*np.amax(mg.node_y) - initial_slope*mg.node_y
        mg['node'][ 'planet_surface__elevation'] = z #+ np.random.rand(len(z))/10000.
    
        craters_component.grid = mg
    
        mg = craters_component.excavate_a_crater_furbish(mg)
        mass_balance.append(craters_component.mass_balance)
        beta.append(craters_component.impact_angle_to_normal)
        
    list_of_mass_bals += list(mass_balance)
                
    fitParams, fitCovariances = curve_fit(fitFunc, work_with, mass_balance)
        
    param_collection = np.vstack((param_collection, fitParams))
    
    synthetic_solution = fitFunc(work_with, fitParams[0], fitParams[1], fitParams[2], fitParams[3], fitParams[4], fitParams[5], fitParams[6])
    #first_synthetic_solution = fitFunc(slope_values, params_from_first_try[0], params_from_first_try[1], params_from_first_try[2], params_from_first_try[3], params_from_first_try[4], params_from_first_try[5], params_from_first_try[6])
    
    print('Done ', i)
    
    pylab.plot(mass_balance)
    pylab.plot(synthetic_solution)
    #pylab.show()

fitParams, fitCovariances = curve_fit(fitFunc, np.tile(work_with, repeats), list_of_mass_bals)

#fitParams = np.mean(param_collection, axis=0)

aggregate_solution = fitFunc(work_with, fitParams[0], fitParams[1], fitParams[2], fitParams[3], fitParams[4], fitParams[5], fitParams[6])    

pylab.plot(aggregate_solution, 'x')


#! /usr/bin/env python
"""


"""

from landlab.components.flexure import FlexureComponent
from landlab import RasterModelGrid


def add_load_to_middle_of_grid(grid, load):
    shape = grid.shape

    load_array = grid.field_values(
        'node', 'lithosphere__overlying_pressure').view()
    load_array.shape = shape
    load_array[shape[0] / 2, shape[1] / 2] = load


def main():
    (n_rows, n_cols) = (100, 100)
    (dy, dx) = (10e3, 10e3)

    grid = RasterModelGrid(n_rows, n_cols, dx)

    flex = FlexureComponent(grid, method='flexure')

    add_load_to_middle_of_grid(grid, 1e9)

    flex.update()

    grid.imshow('node', 'lithosphere__elevation', symmetric_cbar=True,
                show=True) 


if __name__ == '__main__':
    main()



import numpy as np
from numpy.testing import assert_array_equal
from nose import with_setup
from nose.tools import assert_is, assert_equal

from landlab.grid import raster_funcs as rfuncs


def setup_unit_grid():
    from landlab import RasterModelGrid
    globals().update({
        'rmg': RasterModelGrid(4, 5),
        'values_at_nodes':  np.arange(20.),
    })


def setup_grid():
    from landlab import RasterModelGrid
    globals().update({
        'rmg': RasterModelGrid(4, 5, 2.),
        'values_at_nodes':  np.arange(20.),
    })


@with_setup(setup_unit_grid)
def test_scalar_arg():
    grads = rfuncs.calculate_gradient_across_cell_faces(
        rmg, values_at_nodes, 0)
    assert_array_equal(grads, np.array([[-1., -5., 1., 5.]]))


@with_setup(setup_unit_grid)
def test_iterable():
    grads = rmg.calculate_gradient_across_cell_faces(values_at_nodes, [0, 4])
    assert_array_equal(grads, np.array([[-1., -5., 1., 5.],
                                        [-1., -5., 1., 5.]]))


@with_setup(setup_unit_grid)
def test_with_no_cell_id_arg():
    values = np.array([0, 1,  3, 6, 10,
                       0, 1,  3, 6, 10,
                       0, 1,  3, 5, 10,
                       0, 1, -3, 6, 10], dtype=float)
    grads = rmg.calculate_gradient_across_cell_faces(values)

    assert_array_equal(grads, np.array([
        [-2., 0., 1., 0.], [-3., 0., 2., 0.], [-4.,  1., 3.,  0.],
        [-2., 0., 1., 0.], [-2., 6., 2., 0.], [-5., -1., 2., -1.]]))


@with_setup(setup_unit_grid)
def test_with_out_keyword():
    out = np.empty((1, 4))
    rtn = rmg.calculate_gradient_across_cell_faces(values_at_nodes, 5, out=out)
    assert_is(rtn, out)
    assert_array_equal(out, np.array([[-1., -5., 1., 5.]]))



#! /usr/env/python
"""

2D numerical model of shallow-water flow over topography, using the
Bates et al. (2010) algorithm for storage-cell inundation modeling.

Last updated GT July 2013

"""

from landlab import RasterModelGrid
import pylab, time
import numpy as np

def main():
    """
    In this simple tutorial example, the main function does all the work: 
    it sets the parameter values, creates and initializes a grid, sets up 
    the state variables, runs the main loop, and cleans up.
    """
    
    # INITIALIZE
    
    # User-defined parameter values
    numrows = 20
    numcols = 100
    dx = 50.
    n = 0.03              # roughness coefficient
    run_time = 1800       # duration of run, seconds
    h_init = 0.001        # initial thin layer of water (m)
    h_boundary = 2.5      # water depth at left side (m) 
    g = 9.8
    alpha = 0.2           # time-step factor (ND; from Bates et al., 2010)
    
    # Derived parameters
    ten_thirds = 10./3.   # pre-calculate 10/3 for speed
    elapsed_time = 0.0    # total time in simulation
    report_interval = 2.  # interval to report progress (seconds)
    next_report = time.time()+report_interval   # next time to report progress
    
    # Create and initialize a raster model grid
    mg = RasterModelGrid(numrows, numcols, dx)
    
    # Set up boundaries. We'll have the right and left sides open, the top and
    # bottom closed. The water depth on the left will be 5 m, and on the right 
    # just 1 mm.
    mg.set_inactive_boundaries(True, False, True, False)
    
    # Set up scalar values
    z = mg.create_node_array_zeros()   # land elevation
    h = mg.create_node_array_zeros() + h_init     # water depth (m)
    q = mg.create_active_link_array_zeros()  # unit discharge (m2/s)
    dhdt = mg.create_node_array_zeros()  # rate of water-depth change
    
    # Left side has deep water
    leftside = mg.left_edge_node_ids()
    h[leftside] = h_boundary
    
    # Get a list of the interior cells
    interior_cells = mg.get_active_cell_node_ids()

    # Display a message
    print( 'Running ...' )
    start_time = time.time()

    # RUN
    
    # Main loop
    while elapsed_time < run_time:
        
        # Report progress
        if time.time()>=next_report:
            print('Time = '+str(elapsed_time)+' ('
                  +str(100.*elapsed_time/run_time)+'%)')
            next_report += report_interval
        
        # Calculate time-step size for this iteration (Bates et al., eq 14)
        dtmax = alpha*mg.dx/np.sqrt(g*np.amax(h))
        
        # Calculate the effective flow depth at active links. Bates et al. 2010
        # recommend using the difference between the highest water-surface
        # and the highest bed elevation between each pair of cells.
        zmax = mg.max_of_link_end_node_values(z)
        w = h+z   # water-surface height
        wmax = mg.max_of_link_end_node_values(w)
        hflow = wmax - zmax
        
        # Calculate water-surface slopes
        water_surface_slope = mg.calculate_gradients_at_active_links(w)
       
        # Calculate the unit discharges (Bates et al., eq 11)
        q = (q-g*hflow*dtmax*water_surface_slope)/ \
            (1.+g*hflow*dtmax*n*n*abs(q)/(hflow**ten_thirds))
        
        # Calculate water-flux divergence and time rate of change of water depth
        # at nodes
        dhdt = -mg.calculate_flux_divergence_at_nodes(q)
        
        # Second time-step limiter (experimental): make sure you don't allow
        # water-depth to go negative
        if np.amin(dhdt) < 0.:
            shallowing_locations = np.where(dhdt<0.)
            time_to_drain = -h[shallowing_locations]/dhdt[shallowing_locations]
            dtmax2 = alpha*np.amin(time_to_drain)
            dt = np.min([dtmax, dtmax2])
        else:
            dt = dtmax
        
        # Update the water-depth field
        h[interior_cells] = h[interior_cells] + dhdt[interior_cells]*dt
        
        # Update current time
        elapsed_time += dt

      
    # FINALIZE
    
    # Get a 2D array version of the elevations
    hr = mg.node_vector_to_raster(h, flip_vertically=True)
    
    # Create a shaded image
    pylab.close()  # clear any pre-existing plot
    image_extent = [0, 0.001*dx*numcols, 0, 0.001*dx*numrows] # in km
    im = pylab.imshow(hr, cmap=pylab.cm.RdBu, extent=image_extent)
    pylab.xlabel('Distance (km)', fontsize=12)
    pylab.ylabel('Distance (km)', fontsize=12)
    
    # add contour lines with labels
    cset = pylab.contour(hr, extent=image_extent)
    pylab.clabel(cset, inline=True, fmt='%1.1f', fontsize=10)
    
    # add a color bar on the side
    cb = pylab.colorbar(im)
    cb.set_label('Water depth (m)', fontsize=12)
    
    # add a title
    pylab.title('Simulated inundation')

    # Display the plot
    pylab.show()
    print('Done.')
    print('Total run time = '+str(time.time()-start_time)+' seconds.')

if __name__ == "__main__":
    main()



#! /usr/bin/env python

import unittest

from landlab.field.grouped import (is_regular_method_name,
                                   get_method_from_class,
                                   prepend_arg_list_with_dict_value,
                                   regular_method_names)


class TestRegularMethodName(unittest.TestCase):
    def test_special_name(self):
        self.assertFalse(is_regular_method_name('__init__'))

    def test_regular_name(self):
        self.assertTrue(is_regular_method_name('do_something'))

    def test_private_name(self):
        self.assertTrue(is_regular_method_name('_do_something_privately'))


class TestRegularMethods(unittest.TestCase):
    def test_no_regulars(self):
        class A(object):
            pass
        self.assertSetEqual(set(), regular_method_names(A))

    def test_one_regular(self):
        class A(object):
            def do_something(self):
                pass
        self.assertSetEqual(set(['do_something']), regular_method_names(A))

    def test_multiple_regular(self):
        class A(object):
            def do_something(self):
                pass
            def do_something_else(self):
                pass
            def stop(self):
                pass
        self.assertSetEqual(set(['do_something', 'do_something_else', 'stop']),
                            regular_method_names(A))

    def test_staticmethod(self):
        class A(object):
            @staticmethod
            def do_something(self):
                pass
        self.assertSetEqual(set(), regular_method_names(A))

    def test_attribute(self):
        class A(object):
            do_something = 'nothing'
        self.assertSetEqual(set(), regular_method_names(A))


class TestGetMethodFromClass(unittest.TestCase):
    def test_exists(self):
        class A(object):
            def do_something(self):
                pass
        self.assertEqual(A.do_something,
                         get_method_from_class(A, 'do_something'))

    def test_absent(self):
        class A(object):
            pass
        with self.assertRaises(AttributeError):
            get_method_from_class(A, 'do_something')

    def test_is_not_method(self):
        class A(object):
            do_something = False
        with self.assertRaises(TypeError):
            get_method_from_class(A, 'do_something')

class TestPrependArgList(unittest.TestCase):
    def test_(self):
        class A(dict):
            pass
        def func():
            pass
        setattr(A, 'do_something', prepend_arg_list_with_dict_value(A, func))
        self.assertTrue(hasattr(A, 'do_something'))



#! /usr/env/python

"""
route_flow_dn.py:
    
Given a ModelGrid, calculates single-path (steepest direction) flow directions,
drainage area, and (optionally) discharge.

The "dn" in the name means that this is a generalization of the D8 algorithm,
for a grid in which a node has N neighbors (N might happen to be 8, or not).

Created GT Nov 2013
"""

import landlab
#from landlab import RasterModelGrid
from landlab.components.flow_routing import flow_direction_DN
reload(flow_direction_DN)
from landlab.components.flow_accum import flow_accum_bw
import numpy


class FlowRouter():
    
    def __init__(self, model_grid):
        
        # We keep a local reference to the grid
        self._grid = model_grid
        
        # We'll also keep track of the active links; if raster, then these are
        # the "D8" links; otherwise, it's just activelinks
        if type(model_grid) is landlab.grid.raster.RasterModelGrid:
            dal, d8f, d8t = model_grid.d8_active_links()
            self._activelink_from = d8f
            self._activelink_to = d8t
        else:
            self._activelink_from = model_grid.activelink_fromnode
            self._activelink_to = model_grid.activelink_tonode
            
        # Keep track of the following variables:
        #   - drainage area at each node
        #   - receiver of each node
        self.drainage_area = model_grid.add_zeros('node', 'drainage_area')
        self.receiver = model_grid.create_node_array_zeros('receiver')
        self.steepest_slope = model_grid.create_node_array_zeros('steepest_slope')
        
    def route_flow(self, elevs, node_cell_area=1.0, runoff_rate=1.0,
                   boundary_nodes=None):
        """
        Routes surface-water flow by (1) assigning to each node a single 
        drainage direction, and then (2) adding up the number of nodes that
        contribute flow to each node on the grid (including the node itself).
        If a scalar is specified for cell_area, computes the total surface 
        contributing area by assuming that each cell has the same surface area.
        If an array is given (with length equal to the number of nodes), these
        areas are used for each cell. Likewise, runoff_rate, in length per
        time (volume per area per time) may be given either as a scalar
        (identical for each cell) or as an array whose length is the number of
        nodes in the grid.
        
        Returns:
            - Node array of receivers (nodes that receive flow)
            - Node array of drainage areas
            - Node array of discharges
            - Node array of steepest downhill slopes
            - Node array containing downstream-to-upstream ordered list of node
              IDs
            - Node array containing ID of link that leads from each node to its
              receiver (or UNDEFINED_INDEX if there is no receiver)
        
        Example:
            >>> from landlab import RasterModelGrid
            >>> mg = RasterModelGrid(5, 4, 1.0)
            >>> elev = numpy.array([0.,  0.,  0., 0., \
                                 0., 21., 10., 0., \
                                 0., 31., 20., 0., \
                                 0., 32., 30., 0., \
                                 0., 0., 0., 0.])
            >>> mg.set_inactive_boundaries(False, True, True, True)
            >>> fr = FlowRouter(mg)
            >>> r, a, q, ss, s, rl = fr.route_flow(elev)
            >>> r
            array([ 0,  1,  2,  3,  4,  1,  2,  7,  8,  6,  6, 11, 12, 10, 10, 15, 16,
                   17, 18, 19])
            >>> a
            array([ 1.,  2.,  6.,  1.,  1.,  1.,  5.,  1.,  1.,  1.,  3.,  1.,  1.,
                    1.,  1.,  1.,  1.,  1.,  1.,  1.])
            >>> r, a, q, ss, s, rl = fr.route_flow(elev, 10.0)
            >>> a
            array([ 10.,  20.,  60.,  10.,  10.,  10.,  50.,  10.,  10.,  10.,  30.,
                    10.,  10.,  10.,  10.,  10.,  10.,  10.,  10.,  10.])
            >>> cell_areas = 10.0 + numpy.arange(mg.number_of_nodes)
            >>> r, a, q, ss, s, rl = fr.route_flow(elev, cell_areas)
            >>> a
            array([  10.,   26.,  114.,   13.,   14.,   15.,  102.,   17.,   18.,
                     19.,   67.,   21.,   22.,   23.,   24.,   25.,   26.,   27.,
                     28.,   29.])
            >>> runoff_rate = numpy.arange(mg.number_of_nodes)
            >>> r, a, q, ss, s, rl = fr.route_flow(elev, 100.0, runoff_rate)
            >>> q
            array([    0.,   600.,  5400.,   300.,   400.,   500.,  5200.,   700.,
                     800.,   900.,  3700.,  1100.,  1200.,  1300.,  1400.,  1500.,
                    1600.,  1700.,  1800.,  1900.])
            >>> r, a, q, ss, s, rl = fr.route_flow(elev, cell_areas, runoff_rate)
            >>> q
            array([    0.,    86.,  1126.,    39.,    56.,    75.,  1102.,   119.,
                     144.,   171.,   835.,   231.,   264.,   299.,   336.,   375.,
                     416.,   459.,   504.,   551.])
        """
        
        # Calculate the downhill-positive slopes at the d8 active links
        #TODO: generalize to use EITHER D8, if raster, or just active links,
        # otherwise.
        link_slope = -self._grid.calculate_gradients_at_d8_active_links(elevs)
        
        # Find the baselevel nodes
        (baselevel_nodes, ) = numpy.where( self._grid.node_status==1 )

        # Calculate flow directions
        receiver, steepest_slope, sink, recvr_link = \
            flow_direction_DN.flow_directions(elevs, self._activelink_from,
                                         self._activelink_to, link_slope, 
                                         baselevel_nodes)
        
        # TODO: either need a way to calculate and return the *length* of the
        # flow links, OR the caller has to handle the raster / non-raster case.
        
        #print 'sinks:', sink

        # Calculate drainage area, discharge, and ...
        a, q, s = flow_accum_bw.flow_accumulation(receiver, sink,
                                                  node_cell_area, runoff_rate,
                                                  boundary_nodes)

        return receiver, a, q, steepest_slope, s, recvr_link


if __name__ == '__main__':
    import doctest
    doctest.testmod()



"""
This is not yet working. Hopefully updating the D8 flow routing methods in the grid will resolve this.
"""

import numpy as np
from pylab import show, imshow, colorbar, plot
import landlab.grid.raster as raster
from landlab.components.flow_routing import flow_routing_D8
from landlab.components.flow_accum import flow_accumulation
from landlab.components.stream_power import stream_power
from landlab.plot.imshow import imshow_grid
from pylab import show
from copy import copy
reload(flow_routing_D8)
reload(flow_accumulation)
reload(stream_power)
reload(raster)

class data(object):
    '''
        This is where all the whole-grid data lives, as arrays over the various elements of the grid.
        '''
    #Data goes here!!!
    def __init__(self, grid):
        self.elev = grid.zeros() #some data

iterations = 10
tstep = 1.

#Make grid, set the elevs
mg = raster.RasterModelGrid(50, 50, 1.)
mg.set_inactive_boundaries(True, True, True, True)
vectors = data(mg)

#A surface dipping right, plus random noise:
loading_vector = np.linspace(50,1,num=50)
vectors.elev = np.repeat(loading_vector, 50)
vectors.elev += np.random.random_sample(vectors.elev.shape)/10.
vectors.init_elev = copy(vectors.elev)

print vectors.elev.shape
#print vectors.elev.reshape((5,10))

print vectors.elev.shape
print type(vectors.elev)
#print vectors.elev.reshape((5,10))
#print mg.node_status.reshape((5,10))

#Initialize the modules:
dirs = flow_routing_D8.RouteFlowD8(len(vectors.elev))
accum = flow_accumulation.AccumFlow(mg, vectors)
sp = stream_power.StreamPower(mg, vectors, tstep)

#print vectors.elev.reshape((5,10))

#Run the modules:
for i in xrange(iterations):
    print i
    vectors.flowdirs, vectors.node_max_gradients = dirs.calc_flowdirs(mg, vectors.elev)
    try:
        print type(vectors.elev)
    except:
        print 'broke after flowdirs'
    vectors.flowacc = accum.calc_flowacc(mg, vectors)
    try:
        print type(vectors.elev)
    except:
        print 'broke after flowacc'
    vectors.elev = sp.stream_power_erosion(mg, vectors)
    try:
        print type(vectors.elev)
        print vectors.elev.shape
    except:
        print 'broke after sp'
    
#print vectors.elev.reshape((5,10))



########## ET_PriestlyTaylor.py ###############
##
##  This component calculates and updates Priestly Taylor Evapotranspiration.
##
##  Equations from ACSE EWRI report (Jan 2005) are used
##  Inputs are read from the file 'PET_input.txt'
##  Input observed data from .mat file 'Bondville_IL.mat' is also
##  read for comparisions with modeled data.
##
##
## Written by Erkan Istanbulluoglu & Sai Nudurupati, 2013.
###########################################

import os
from numpy import * 
from landlab import ModelParameterDictionary
from math import *
from matplotlib.pyplot import *

_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'PET_input.txt')

_DEFAULT_INPUT_FILE_1 = os.path.join(os.path.dirname(__file__),
                                  'Bondville_IL.mat')

class ET:

    def __init__( self ):
        pass

    def initialize( self ):

        MPD = ModelParameterDictionary()
        MPD.read_from_file( _DEFAULT_INPUT_FILE )

        """ Priestly Taylor Constant """
        self._alpha = MPD.read_float( 'alpha' )
        """ Surface Albedo """
        self._a = MPD.read_float( 'a' )
        """ Latent Heat of Vaporization """
        self._pwhv = MPD.read_float( 'pwhv' )
        """ Psychometric Constant """
        self._y = MPD.read_float( 'y' )
        """ Stefan Boltzmann Constant """
        self._sigma = MPD.read_float( 'sigma' )
        """ Solar Constant """
        self._Gsc = MPD.read_float( 'Gsc' )
        """ Latitude of Location under Study """
        self._lat = MPD.read_float( 'lat' )
        """ Elevation of Location under Study """
        self._z = MPD.read_float( 'z' )
        """ Adjustment Coefficient """
        self._Krs = MPD.read_float( 'Krs' )        

        """ Reading .mat file for comparision of simulations with observed data.
            Observed data is included in .mat file """
        
        MPD.read_from_mat( _DEFAULT_INPUT_FILE_1 )
        """ Day of the year """
        self._Dm = MPD.data['Dm']
        """ Month of the year """
        self._M = MPD.data['M']
        """ Year """
        self._Y = MPD.data['Y']
        """ Maximum temperature of the day """
        self._Tmax = MPD.data['Tmax']
        """ Minimum temperature of the day """
        self._Tmin = MPD.data['Tmin']        
        """ Observed Short Wave Radiation """
        self._RsObs = MPD.data['RsObs']
        
        self._length = len(self._RsObs)

        """ Initialize Arrays """
        """ Saturation Vapor Pressure Gradient """
        self._delta = zeros( self._length, dtype = float )
        """ Saturation Vapor Pressure """
        self._es = zeros( self._length, dtype = float )
        """ Actual Vapor Pressure """
        self._ea = zeros( self._length, dtype = float )
        """ Net Long Wave Radiation """
        self._Rnl = zeros( self._length, dtype = float )
        """ Net Short Wave Radiation """
        self._Rns = zeros( self._length, dtype = float )
        """ Net Radiation """
        self._Rn = zeros( self._length, dtype = float )
        """ Clear Sky Solar Radiation """
        self._Rso = zeros( self._length, dtype = float )
        """ Short Wave Radiation """
        self._Rs = zeros( self._length, dtype = float )        
        self._u = zeros( self._length, dtype = float )
        self._ETp = zeros( self._length, dtype = float )
        """ Extraterrestrial Radiation """
        self._Ra = zeros( self._length, dtype = float )
        """ Julian Day """
        self._J = zeros( self._length, dtype = float )
        self._ws = zeros( self._length, dtype = float )
        self._x = zeros( self._length, dtype = float )
        """ Solar Declination Angle """
        self._sdecl = zeros( self._length, dtype = float )
        self._dr = zeros( self._length, dtype = float )
        self._Tavg = zeros( self._length, dtype = float )
        self._fcd = zeros( self._length, dtype = float )
        self._phi = (3.14/180)*self._lat

           
    def update( self ):

        for i in xrange( 0, self._length ):
            """
                Julian Day - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (25)
            """
            self._J[i] = self._Dm[i] - 32.0 + round(275.0*(self._M[i]/9.0)) + \
                        (2*round(3/(self._M[i]+1))) + round((self._M[i]/100.0)- \
                        (self._Y[i]%4.0)/4.0+0.975)
            """
                Saturation Vapor Pressure - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (37)
            """
            self._es[i] = 0.6108*exp((17.27*self._Tavg[i])/(237.7+self._Tavg[i]))
            """
                Actual Vapor Pressure - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (38)
            """
            self._ea[i] = 0.6108*exp((17.27*self._Tmin[i])/(237.7+self._Tmin[i]))
            """
                Slope of Saturation Vapor Pressure - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (36)
            """
            self._delta[i] = (4098.0*self._es[i])/(pow((237.3+self._Tavg[i]),2.0))
            """
                Solar Declination Angle - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (51)
            """
            self._sdecl[i] = 0.409*sin((((2.0*3.14)/365.0)*self._J[i])-1.39)
            """
                Inverse Relative Distance Factor - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (50)
            """
            self._dr[i] = 1 + (0.033*cos((2.0*3.14/365.0)*self._J[i]))
            """
                To calculate ws - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (61)
            """
            self._x[i] = 1.0-((pow((tan(self._phi)),2.0))*(pow(tan(self._sdecl[i]),2.0)))  
            if self._x[i] <= 0:
                self._x[i] = 0.00001;
                
            """
                Sunset Hour Angle - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (60)
            """
            self._ws[i] = (3.14/2.0)-atan((-1*tan(self._phi)*tan(self._sdecl[i]))/(pow(self._x[i],2.0)))
            """
                Extraterrestrial radmodel.docx - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (48)
            """
            self._Ra[i] = 11.57*(24.0/3.14)*4.92*self._dr[i]*((self._ws[i]*sin(self._phi)*sin(self._sdecl[i]))+ \
                          (cos(self._phi)*cos(self._sdecl[i])*(sin(self._ws[i]))))
            """
                Clear-sky Solar Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (47)
            """
            self._Rso[i] = (0.75+((2.0*pow(10,-5.0))*self._z))*self._Ra[i]
            self._Rs[i] = min(self._Krs*self._Ra[i]*sqrt(self._Tmax[i]-self._Tmin[i]), self._Rso[i])
            """
                Net Short Wave Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (43)
            """
            self._Rns[i] = self._Rs[i]*(1-self._a)

            """
                Relative Cloudiness - ASCE-EWRI Task Committee Report, Jan-2005 - Page 35
            """
            if self._Rso[i] > 0:
                self._u[i] = self._Rs[i]/self._Rso[i]
            else:
                self._u[i] = 0

            if self._u[i] < 0.3:
                self._u[i] = 0.3
            elif self._u[i] > 1:
                self._u[i] = 1.0

            """
                Cloudiness Function - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (45)
            """
            self._fcd[i] = (1.35*self._u[i])-0.35
            """
                Net Long Wave Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (44)
            """
            self._Rnl[i] = self._sigma*self._fcd[i]*(0.34-(0.14*sqrt(self._ea[i]))* \
                           ((pow((self._Tmax[i]+273.16),4.0)+ \
                            pow((self._Tmin[i]+273.16),4.0))/2.0))

            """
                Net Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (42)
            """                                         
            self._Rn[i] = self._Rns[i] - self._Rnl[i]

            self._ETp[i] = max(self._alpha*(self._delta[i]/(self._delta[i]+self._y) \
                           )*(self._Rn[i]/self._pwhv), 0)                                                                                                       


    def plott( self ):

        figure(1)
        self._px = xrange( 1, self._length+1)
        plot( self._px, self._Rs, label = 'Modeled' )
        hold(True)
        plot( self._px, self._RsObs, '*', label = 'Observed' )
        hold(False)
        xlabel( 'Days' )
        ylabel( 'Short Wave Radiation' )
        title( 'Shortwave Radiation' )
        legend()
        

        figure(2)
        plot( self._px, self._ETp )
        xlabel( 'Days' )
        ylabel( 'Evapotranspiration PET' )
        title( 'Evapotranspiration' )
        show()




import numpy as np
from pylab import show, imshow, colorbar, plot
from landlab import RasterModelGrid
from landlab.components.flow_routing.flow_routing_D8 import RouteFlowD8
from landlab.components.flow_accum.flow_accumulation2 import AccumFlow
from landlab.plot.imshow import imshow_grid
from landlab.components.dem_support.dem_boundary_conditions import WatershedBoundaryConditions
from random import uniform
#reload(flow_routing_D8)
#reload(flow_accumulation)
#reload(raster)

def main():
    nr = 5
    nc = 6
    nnodes = nr*nc
    dx=3
    #instantiate grid
    rg = RasterModelGrid(nr, nc, dx)
    #rg.set_inactive_boundaries(False, False, True, True)
    
    nodata_val=-1
    z  = nodata_val*np.ones( nnodes )    
    #set-up interior elevations with random numbers
    #for i in range(0, nnodes):
    #    if rg.is_interior(i):
    #        elevations[i]=random.random_sample()
    
    #set-up with prescribed elevations to test drainage area calcualtion
    helper = [7,8,9,10,13,14,15,16]
    for i in xrange(0, len(helper)):
        #print 'helper[i]', helper[i]
        z[helper[i]]=2+uniform(-0.5,0.5)
    helper = [19,20,21,22]
    for i in xrange(0, len(helper)):
        z[helper[i]]=3+uniform(-0.5,0.5)
        
    z[7]=1
    
    bc=WatershedBoundaryConditions()
    bc.set_bc_find_outlet(rg, z, nodata_val)
    
    #instantiate variable of type RouteFlowD8 Class
    flow_router = RouteFlowD8(len(z))
    #initial flow direction
    flowdirs, max_slopes = flow_router.calc_flowdirs(rg,z)
    #insantiate variable of type AccumFlow Class
    accumulator = AccumFlow(rg)
    #initial flow accumulation
    drain_area = accumulator.calc_flowacc(rg, z, flowdirs)
    
    print "elevations ", rg.node_vector_to_raster(z)
    print "flowdirs ", rg.node_vector_to_raster(flowdirs)
    print "drain_area ", rg.node_vector_to_raster(drain_area)
    
    
if __name__ == '__main__':
    main()



from numpy.testing import assert_array_equal


class TestActiveLinkGradients(object):
    def setup_grids(self):
        self.grids = {
            'unit_spacing': RasterModelGrid(4, 5),
            'non_unit_spacing': RasterModelGrid(4, 5, 2.),
        }

    def test_unit_spacing(self):
        (rmg, values_at_nodes) = RasterModelGrid(4, 5), np.arange(20.)
        grads = rmg.calculate_gradients_at_active_links(values)

        assert_array_equal(grads, np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                            1, 1, 1, 1, 1, 1, 1, 1]))

        diffs = rmg.calculate_diff_at_active_links(values)
        assert_array_equal(grads, diffs)
        
    def test_non_unit_spacing(self):
        _SPACING = 2.
        (rmg, values_at_nodes) = RasterModelGrid(4, 5, _SPACING), np.arange(20.)
        grads = rmg.calculate_gradients_at_active_links(values)
        assert_array_equal(grads, (1. / _SPACING) *
                              np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                        1, 1, 1, 1, 1, 1, 1, 1]))
        diffs = rmg.calculate_diff_at_active_links(values)
        assert_array_equal(grads, (1. / _SPACING) * diffs)

    def test_out_array(self):
        (rmg, values_at_nodes) = RasterModelGrid(4, 5), np.arange(20.)
        grads = np.empty(17)
        rtn_grads = rmg.calculate_gradients_at_active_links(values,
                                                            out=grads)
        assert_array_equal(grads, np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                                               1, 1, 1, 1, 1, 1, 1, 1]))
        self.assertIs(rtn_grads, grads)
        
    def test_diff_out_array(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20)
        diff = np.empty(17)
        rtn_diff = rmg.calculate_diff_at_active_links(values, out=diff)
        assert_array_equal(
            diff,
            np.array([5, 5, 5, 5, 5, 5, 5, 5, 5,
                      1, 1, 1, 1, 1, 1, 1, 1]))
        self.assertIs(rtn_diff, diff)


class TestLinkGradients(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_unit_spacing(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20, dtype=float)
        grads = rmg.calculate_gradients_at_links(values)
        assert_array_equal(
            grads,
            np.array([5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                     dtype=float))
        diffs = rmg.calculate_diff_at_links(values)
        assert_array_equal(grads, diffs)
        
    def test_non_unit_spacing(self):
        rmg = RasterModelGrid(4, 5, 2.)
        values = np.arange(20, dtype=float)
        grads = rmg.calculate_gradients_at_links(values)
        assert_array_equal(
            grads,
            .5 * np.array([5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                           1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                          dtype=float))
        diffs = rmg.calculate_diff_at_links(values)
        assert_array_equal(grads, .5 * diffs)

    def test_out_array(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20, dtype=float)
        grads = np.empty(31.)
        rtn_grads = rmg.calculate_gradients_at_links(values, out=grads)
        assert_array_equal(
            grads,
            np.array([5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                     dtype=float))
        self.assertIs(rtn_grads, grads)

    def test_diff_out_array(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20, dtype=float)
        diff = np.empty(31.)
        rtn_diff = rmg.calculate_diff_at_links(values, out=diff)
        assert_array_equal(
            diff,
            np.array([5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
                      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
                     dtype=float))
        self.assertIs(rtn_diff, diff)
        
        
class TestMaxGradients(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_scalar_arg(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20, dtype=float)
        grad = rfuncs.calculate_max_gradient_across_cell_faces(rmg, values, 0)
        self.assertEqual(grad, 5.)

    def test_iterable(self):
        rmg = RasterModelGrid(4, 5)
        values = np.arange(20, dtype=float)
        grad = rfuncs.calculate_max_gradient_across_cell_faces(rmg, values, [0, 4])
        assert_array_equal(grad, [5., 5.])

    def test_scalar_arg_with_links(self):
        rmg = RasterModelGrid(4, 5)
        values = np.array([0, 1,  3, 6, 10,
                           0, 1,  3, 6, 10,
                           0, 1,  3, 5, 10,
                           0, 1, -3, 6, 10,])
        (grad, face) = rfuncs.calculate_max_gradient_across_cell_faces(
            rmg, values, (0, 4), return_face=True)
        assert_array_equal(grad, [1, 6])
        assert_array_equal(face, [1, 2])

        link_ids = rfuncs.active_link_id_of_cell_neighbor(rmg, [0, 4], face)
        assert_array_equal(link_ids, [9, 7])

        node_ids = rfuncs.node_id_of_cell_neighbor(rmg, [0, 4], face)
        assert_array_equal(node_ids, [5, 17])

class TestGradientsAcrossFaces(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_no_arg(self):
        rmg = RasterModelGrid(4, 5)
        values = np.array([0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,])
        grads = rfuncs.calculate_gradients_across_cell_faces(rmg, values)
        assert_array_equal(
            grads,
            np.array([[-2, 0, 1, 0], [-3, 0, 2, 0], [-4, 0, 3, 0],
                      [-2, 0, 1, 0], [-3, 0, 2, 0], [-4, 0, 3, 0]]))

    def test_scalar_arg(self):
        rmg = RasterModelGrid(4, 5)
        values = np.array([0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,])
        grads = rfuncs.calculate_gradients_across_cell_faces(rmg, values, 0)
        assert_array_equal(grads, np.array([[-2, 0, 1, 0]]))

    def test_iterable_arg(self):
        rmg = RasterModelGrid(4, 5)
        values = np.array([0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,
                           0, 1, 3, 6, 10,])
        grads = rfuncs.calculate_gradients_across_cell_faces(
            rmg, values, np.array([0, 4]))
        assert_array_equal(
            grads, np.array([[-2, 0, 1, 0], [-3, 0, 2, 0]]))


class TestFluxDivergence(unittest.TestCase, NumpyArrayTestingMixIn):
    def test_flux_from_south_to_north(self):
        rmg = RasterModelGrid(4, 5)
        active_link_flux = np.array([0., 0., 0., 1., 1., 1., 3., 3., 3.,
                                     0., 0., 0., 0., 0., 0., 0., 0.])
        divs = rmg.calculate_flux_divergence_at_nodes(active_link_flux)

        assert_array_equal(
            divs,
            np.array([0.,  0.,  0.,  0., 0.,
                      0.,  1.,  1.,  1., 0.,
                      0.,  2.,  2.,  2., 0.,
                      0., -3., -3., -3., 0.]))
        
    def test_flux_from_east_to_west(self):
        rmg = RasterModelGrid(4, 5)
        active_link_flux = np.array([0., 0., 0., 0., 0., 0., 0., 0., 0.,
                                     0., 1., 3., 6., 0., 1., 3., 6.])
        divs = rmg.calculate_flux_divergence_at_nodes(active_link_flux)

        assert_array_equal(
            divs,
            np.array([0., 0., 0., 0.,  0.,
                      0., 1., 2., 3., -6.,
                      0., 1., 2., 3., -6.,
                      0., 0., 0., 0.,  0.]))
        
    def test_out_array(self):
        rmg = RasterModelGrid(4, 5)
        active_link_flux = np.array([0., 0., 0., 0., 0., 0., 0., 0., 0.,
                                     0., 1., 3., 6., 0., 1., 3., 6.])

        divs = np.empty(20)
        rtn_divs = rmg.calculate_flux_divergence_at_nodes(active_link_flux,
                                                          out=divs)

        assert_array_equal(
            divs,
            np.array([0., 0., 0., 0.,  0.,
                      0., 1., 2., 3., -6.,
                      0., 1., 2., 3., -6.,
                      0., 0., 0., 0.,  0.]))
        self.assertIs(rtn_divs, divs)





#! /usr/bin/env python

import numpy as np
try:
    import matplotlib.pyplot as plt
except ImportError:
    import warnings
    warnings.warn('matplotlib not found', ImportWarning)


def assert_array_size_matches(array, size, msg=None):
    if size != array.size:
        if msg is None:
            msg = '%d != %d' % (size, array.size)
        raise ValueError(msg)


def imshow_node_grid(grid, values, **kwds):
    assert_array_size_matches(values, grid.number_of_nodes,
            'number of values does not match number of nodes')

    data = values.view()
    data.shape = grid.shape

    _imshow_grid_values(grid, data, **kwds)


def imshow_active_node_grid(grid, values, **kwds):
    active_nodes = grid.active_nodes
    assert_array_size_matches(values, active_nodes.size,
            'number of values does not match number of active nodes')
    
    data = np.zeros(grid.number_of_nodes)
    data[active_nodes] = values.flat
    data.shape = grid.shape

    _imshow_grid_values(grid, data, **kwds)


def imshow_cell_grid(grid, values, **kwds):
    assert_array_size_matches(values, grid.number_of_cells,
            'number of values does not match number of cells')

    data = values.view()
    data.shape = (grid.shape[0] - 2, grid.shape[1] - 2)

    _imshow_grid_values(grid, data, **kwds)


def imshow_active_cell_grid(grid, values, **kwds):
    active_cells = grid.node_index_at_active_cells
    assert_array_size_matches(values, active_cells.size,
            'number of values does not match number of active cells')

    data = np.zeros(grid.number_of_nodes)
    data[active_cells] = values
    data.shape = grid.shape

    _imshow_grid_values(grid, data, **kwds)


def _imshow_grid_values(grid, values, var_name=None, var_units=None,
                        grid_units=(None, None), symmetric_cbar=False,
                        cmap='jet', limits=None):
    if len(values.shape) != 2:
        raise ValueError('dimension of values must be 2 (%s)' % values.shape)

    y = np.arange(values.shape[0] + 1) - grid.dx * .5
    x = np.arange(values.shape[1] + 1) - grid.dx * .5

    kwds = dict(cmap=cmap)
    if limits is None:
        if symmetric_cbar:
            (var_min, var_max) = (values.min(), values.max())
            limit = max(abs(var_min), abs(var_max))
            (kwds['vmin'], kwds['vmax']) = (- limit, limit)
    else:
        (kwds['vmin'], kwds['vmax']) = (limits[0], limits[1])


    plt.pcolormesh(x, y, values, **kwds)

    plt.gca().set_aspect(1.)
    plt.autoscale(tight=True)

    plt.colorbar()

    plt.xlabel('X (%s)' % grid_units[1])
    plt.ylabel('Y (%s)' % grid_units[0])

    if var_name is not None:
        plt.title('%s (%s)' % (var_name, var_units))

    #plt.show()


def imshow_grid(grid, values, **kwds):
    show = kwds.pop('show', False)
    values_at = kwds.pop('values_at', 'node')

    if values_at == 'node':
        imshow_node_grid(grid, values, **kwds)
    elif values_at == 'cell':
        imshow_cell_grid(grid, values, **kwds)
    elif values_at == 'active_node':
        imshow_active_node_grid(grid, values, **kwds)
    elif values_at == 'active_cell':
        imshow_active_cell_grid(grid, values, **kwds)
    else:
        raise TypeError('value location %s not understood' % values_at)

    if show:
        plt.show()


def imshow_field(field, name, **kwds):
    values_at = kwds['values_at']
    imshow_grid(field, field.field_values(values_at, name), var_name=name,
                var_units=field.field_units(values_at, name), **kwds)

###
# Added by Sai Nudurupati 29Oct2013 
# This function is exactly the same as imshow_grid but this function plots
# arrays spread over cells rather than nodes

def imshow_active_cells(grid, values, var_name=None, var_units=None,
                grid_units=(None, None), symmetric_cbar=False,
                cmap='jet'):
    data = values.view()
    data.shape = (grid.shape[0]-2, grid.shape[1]-2)

    y = np.arange(data.shape[0]) - grid.dx * .5
    x = np.arange(data.shape[1]) - grid.dx * .5

    if symmetric_cbar:
        (var_min, var_max) = (data.min(), data.max())
        limit = max(abs(var_min), abs(var_max))
        limits = (-limit, limit)
    else:
        limits = (None, None)

    plt.pcolormesh(x, y, data, vmin=limits[0], vmax=limits[1], cmap=cmap)

    plt.gca().set_aspect(1.)
    plt.autoscale(tight=True)

    plt.colorbar()

    plt.xlabel('X (%s)' % grid_units[1])
    plt.ylabel('Y (%s)' % grid_units[0])

    if var_name is not None:
        plt.title('%s (%s)' % (var_name, var_units))

    plt.show()

###



import numpy as np
import numpy.ma as ma

"""
Preliminary work for a submarine delta foreset sed router. Not yet functioning!
DEJH, Sept 2013
"""

class Littoral(object):
    def __init__(self, grid, dry_land_Qs_array, elev):
        #dry_land_Qs_array is a np array of -1s for any cell <SL, with Qs for any dry land cell.
        #Very important that when sed is routed on dry land, sea cells act as hard boundaries, BC=4.
        self.dry_land_Qs = dry_land_Qs_array[dry_land_Qs_array>0]
        
        
    def assign_Qs_to_littoral_cells(self, grid, dry_land_Qs_array, elev):
        self.dry_land_Qs = dry_land_Qs_array[dry_land_Qs_array>0]
        cell_neighbors = grid.get_neighbor_list(self.dry_land_Qs)
        cell_neighbor_heights = elev[cell_neighbors]
        number_wet_cell_neighbors = np.sum(cell_neighbor_heights<0, axis=1)
        wet_neighbor_nonzero = np.where(number_wet_cell_neighbors>0)
        wet_neighbor_count_nonzero = number_wet_cell_neighbors[wet_neighbor_nonzero]
        Qs_to_each_neighbor = dry_land_Qs_array[wet_neighbor_nonzero]/wet_neighbor_count_nonzero
        wet_cell_neighbors = np.where(cell_neighbor_heights<0,cell_neighbors,-1)[wet_neighbor_nonzero]
        self.Qs_in_source_cell_extended = np.zeros(len(dry_land_Qs_array)+1)
        self.Qs_in_source_cell_extended[wet_cell_neighbors] += Qs_to_each_neighbor #broadcasting should sort this out automatically
        self.Qs_in_source_cell = self.Qs_in_source_cell_extended[:-1]
        
        return self.Qs_in_source_cell
        #This returned array is filled with zeros, except where sed has been discharged directly into the sea cells, where it
        #is equal to the total sed flux entering that cell.

class ForesetAggrade(object):
    def __init__(self, grid, source_cells_Qs, elev):
        self.tan_repose_angle = np.tan(32.*np.pi/180)
        #source_cells_Qs is a np array of 0s except where a (submarine) source cell, when it contains the Qs delivered to that cell from land.
        
    def aggrade_front(self, grid, tstep, source_cells_Qs, elev, SL): #ensure Qs and tstep units match!
        
        self.total_sed_supplied_in_tstep = source_cells_Qs*tstep
        self.Qs_sort_order = np.argsort(source_cells_Qs)[::-1] #descending order
        self.Qs_sort_order = self.Qs_sort_order[:np.count_nonzero(self.Qs_sort_order>0)]
        for i in self.Qs_sort_order:
            subaerial_nodes = elev>=SL
            subsurface_elev_array = ma.array(elev, subaerial_nodes)
            xy_tuple = (grid.node_x[i], grid.node_y[i])
            distance_map = grid.get_distances_of_nodes_to_point(xy_tuple)
            loop_number = 0
            closest_node_list = ma.argsort(ma.masked_array(distance_map, mask=subsurface_elev_array.mask))
            smooth_cone_elev_from_apex = subsurface_elev_array[i]-distance_map*self.tan_repose_angle
            while 1:
                filled_all_cells_flag = 0
                accom_space_at_controlling_node = SL - subsurface_elev_array[closest_node_list[loop_number]]
                new_max_cone_surface_elev = smooth_cone_elev_from_apex + accom_space_at_controlling_node
                subsurface_elev_array.mask = (elev>=SL or new_max_cone_surface_elev<elev)
                depth_of_accom_space = new_max_cone_surface_elev - subsurface_elev_array
                accom_depth_order = ma.argsort(depth_of_accom_space)[::-1]
                #Vectorised method to calc fill volumes:
                area_to_fill = ma.cumsum(grid.cellarea[accom_depth_order])
                differential_depths = ma.empty_like(depth_of_accom_space)
                differential_depths[:-1] = depth_of_accom_space[accom_depth_order[:-1]] - depth_of_accom_space[accom_depth_order[1:]]
                differential_depths[-1] = depth_of_accom_space[accom_depth_order[-1]]
                incremental_volumes = ma.cumsum(differential_depths*area_to_fill)
                match_position_of_Qs_in = ma.searchsorted(incremental_volumes, self.total_sed_supplied_in_tstep[i])
                try:
                    depths_to_add = depth_of_accom_space-depth_of_accom_space[match_position_of_Qs_in]
                except:
                    depths_to_add = depth_of_accom_space-depth_of_accom_space[match_position_of_Qs_in-1]
                    filled_all_cells_flag = 1
                depths_to_add = depths_to_add[ma.where(depths_to_add>=0)]
                if not filled_all_cells_flag:
                    depths_to_add += (self.total_sed_supplied_in_tstep[i] - incremental_volumes[match_position_of_Qs_in-1])/area_to_fill[match_position_of_Qs_in-1]
                    subsurface_elev_array[accom_depth_order[len(depths_to_add)]] = depths_to_add
                    self.total_sed_supplied_in_tstep[i] = 0
                    break
                else:
                    subsurface_elev_array[accom_depth_order] = depths_to_add
                    self.total_sed_supplied_in_tstep[i] -= incremental_volumes[-1]
                    loop_number += 1
        
        return elev



#! /usr/bin/env python

import numpy as np


_UNKNOWN_UNITS = '?'


class ScalarDataFields(dict):
    def __init__(self, size):
        self._size = size

        #print 'ScalarDataFields.__init__'
        super(ScalarDataFields, self).__init__()
        self._units = dict()

    @property
    def units(self):
        return self._units

    @property
    def size(self):
        return self._size

    @property
    def number_of_values(self):
        return self._size

    def empty(self, **kwds):
        """
        Return a new array of the data field size, without initializing
        entries. Keyword arguments are the same as that for the equivalent
        numpy function.
        """
        return np.empty(self.size, **kwds)

    def ones(self, **kwds):
        """
        Return a new array of the data field size, filled with ones. Keyword
        arguments are the same as that for the equivalent numpy function.
        """
        return np.ones(self.size, **kwds)

    def zeros(self, **kwds):
        """
        Return a new array of the data field size, filled with zeros. Keyword
        arguments are the same as that for the equivalent numpy function.
        """
        return np.zeros(self.size, **kwds)

    def add_empty(self, name, units=_UNKNOWN_UNITS, **kwds):
        """
        Create a new array of the data field size, without initializing
        entries, and add it to the field as *name*. The *units* keyword gives
        the units of the new fields as a string. Remaining keyword arguments
        are the same as that for the equivalent numpy function.

        Returns a reference to the newly-created array.
        """
        return self.add_field(name, self.empty(**kwds), units=units)

    def add_ones(self, name, units=_UNKNOWN_UNITS, **kwds):
        """
        Create a new array of the data field size, filled with ones, and
        add it to the field as *name*. The *units* keyword gives the units of
        the new fields as a string. Remaining keyword arguments are the same
        as that for the equivalent numpy function.

        Returns a reference to the newly-created array.
        """
        return self.add_field(name, self.ones(**kwds), units=units)

    def add_zeros(self, name, units=_UNKNOWN_UNITS, **kwds):
        """
        Create a new array of the data field size, filled with zeros, and
        add it to the field as *name*. The *units* keyword gives the units of
        the new fields as a string. Remaining keyword arguments are the same
        as that for the equivalent numpy function.

        Returns a reference to the newly-created array.
        """
        return self.add_field(name, self.zeros(**kwds), units=units)

    def add_field(self, name, value_array, units=_UNKNOWN_UNITS, copy=False):
        if copy:
            value_array = value_array.copy()

        self[name] = value_array

        self.set_units(name, units)
        return self[name]

    def set_units(self, name, units):
        self._units[name] = units

    def __setitem__(self, name, value_array):
        assert(value_array.size == self.size)

        if name not in self:
            self.set_units(name, None)

        super(ScalarDataFields, self).__setitem__(name, value_array)



#! /usr/bin/env python

import numpy

from .voronoi import VoronoiDelaunayGrid


class RadialModelGrid(VoronoiDelaunayGrid):
    """
    This inherited class implements a circular grid in which grid nodes are
    placed at regular radial and semi-regular arc-wise intervals. That is, if
    the radial spacing between "shells" is dr, the nodes are placed around the
    circular shell at regular intervals that get as close as possible to dr.
    The points are then arranged in a Delaunay triangulation with Voronoi cells.
    
   # Examples:
   #     
   #     >>> hmg = HexModelGrid(3, 2, 1.0)
   #     >>> hmg.number_of_nodes
   #     7
    """
    
    def __init__(self, num_shells=0, dr=1.0, origin_x=0.0, origin_y=0.0, **kwds):
        """
        Optionally takes number of shells and radial distance between shells. 
        If this are given, calls initialize() to set up the grid.
        
        """
        #print 'RadialModelGrid.init'
        
        # Set number of nodes, and initialize if caller has given dimensions
        #self._num_nodes = num_rows * num_cols
        if num_shells > 0:
            self._initialize(num_shells, dr, origin_x, origin_y)
        super(RadialModelGrid, self).__init__(**kwds)

    def _initialize( self, num_shells, dr, origin_x=0.0, origin_y=0.0):
        """
        """
        #if self.DEBUG_TRACK_METHODS:
        print 'RadialModelGrid._initialize('+str(num_shells)+', '+str(dr)+')'
        
        [pts, npts] = self.make_radial_points(num_shells, dr)
        super(RadialModelGrid, self)._initialize(pts[:,0], pts[:,1])
        
        
    def make_radial_points(self, num_shells, dr, origin_x=0.0, origin_y=0.0):
        """
        Creates and returns a set of (x,y) points placed in a series of
        concentric circles around the origin.
        """
        shells = numpy.arange(0, num_shells)+1
        twopi = 2*numpy.pi
        n_pts_in_shell = numpy.round( twopi*shells ) # number of points in each shell
        dtheta = twopi / n_pts_in_shell
        npts = int(sum(n_pts_in_shell)+1)
        pts = numpy.zeros((npts,2))
        r =  shells*dr
        startpt = 1
        for i in numpy.arange(0, num_shells):
            theta = dtheta[i]*numpy.arange(0, n_pts_in_shell[i])+dtheta[i]/(i+1)
            pts[startpt:(startpt+n_pts_in_shell[i]),0] = r[i]*numpy.cos(theta)
            pts[startpt:(startpt+n_pts_in_shell[i]),1] = r[i]*numpy.sin(theta)
            startpt += n_pts_in_shell[i]
        pts[:,0] += origin_x
        pts[:,1] += origin_y
        
        return pts, npts



#! /usr/bin/env python

import unittest
import numpy as np
from numpy.testing import assert_array_equal

from landlab.field.grouped import ModelDataFields


class TestModelDataFields(unittest.TestCase):
    def test_init(self):
        fields = ModelDataFields()
        self.assertSetEqual(set(), fields.groups)

    def test_new_field_location(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        self.assertSetEqual(set(['node']), fields.groups)

    def test_add_existing_group(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        with self.assertRaises(ValueError):
            fields.new_field_location('node', 24)

    def test_add_multiple_groups(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        fields.new_field_location('cell', 2)
        fields.new_field_location('face', 7)
        fields.new_field_location('link', 7)
        self.assertSetEqual(set(['node', 'cell', 'face', 'link']),
                            fields.groups)

    def test_ones(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        fields.new_field_location('cell', 2)

        value_array = fields.ones('node')
        assert_array_equal(np.ones(12), value_array)

        value_array = fields.ones('cell')
        assert_array_equal(np.ones(2), value_array)

    def test_add_ones(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        fields.new_field_location('cell', 2)

        fields.add_ones('node', 'z')
        assert_array_equal(np.ones(12), fields['node']['z'])
        assert_array_equal(np.ones(12), fields.field_values('node', 'z'))

        fields.add_ones('cell', 'z')
        assert_array_equal(np.ones(2), fields['cell']['z'])
        assert_array_equal(np.ones(2), fields.field_values('cell', 'z'))

    def test_add_ones_return_value(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)
        fields.new_field_location('cell', 2)

        rtn_value = fields.add_ones('node', 'z')
        assert_array_equal(rtn_value, np.ones(12))
        self.assertIs(rtn_value, fields['node']['z'])
        self.assertIs(rtn_value, fields.field_values('node', 'z'))

        rtn_value = fields.add_ones('cell', 'z')
        assert_array_equal(rtn_value, np.ones(2))
        self.assertIs(rtn_value, fields['cell']['z'])
        self.assertIs(rtn_value, fields.field_values('cell', 'z'))

    def test_getitem(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)

        self.assertDictEqual(dict(), fields['node'])
        with self.assertRaises(KeyError):
            fields['cell']

    def test_at_attribute(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)

        self.assertDictEqual(dict(), fields.at_node)
        with self.assertRaises(AttributeError):
            fields.at_cell

        fields.add_ones('node', 'z')
        assert_array_equal(np.ones(12), fields.at_node['z'])

    def test_has_group(self):
        fields = ModelDataFields()
        fields.new_field_location('node', 12)

        self.assertTrue(fields.has_group('node'))
        self.assertFalse(fields.has_group('cell'))



from landlab import RasterModelGrid
from landlab import ModelParameterDictionary
from landlab.components.sed_trp_shallow_flow.transport_sed_in_shallow_flow import SurfaceFlowTransport

import time
import pylab
import numpy as np

#get the needed properties to build the grid:
inputs = ModelParameterDictionary('./stof_params.txt')
nrows = inputs.read_int('nrows')
ncols = inputs.read_int('ncols')
dx = inputs.read_float('dx')
h_init = inputs.read_float('h_init')
h_boundary = inputs.read_float('h_boundary')
drop_ht = inputs.read_float('drop_ht')
initial_slope = inputs.read_float('initial_slope')
time_to_run = inputs.read_int('run_time')

left_middle_node = ncols*(nrows/2)
z0 = drop_ht+dx*(ncols-1)*initial_slope

mg = RasterModelGrid(nrows, ncols, dx)
mg.set_inactive_boundaries(True, False, True, False)

#create the fields in the grid
mg.create_node_array_zeros('planet_surface__elevation')
mg.create_node_array_zeros('planet_surface__water_depth')

#set the initial water depths
h = mg.create_node_array_zeros() + h_init
h[left_middle_node] = h_boundary
h[left_middle_node-ncols] = h_boundary
h[left_middle_node+ncols] = h_boundary
mg['node'][ 'planet_surface__water_depth'] = h

# Set initial topography
x = mg.get_node_x_coords()
y = mg.get_node_y_coords()
zinit = mg.create_node_array_zeros()
zinit = z0-initial_slope*x
rightside = mg.right_edge_node_ids()
zinit[rightside] = 0.
mg['node']['planet_surface__elevation'] = zinit

# Display a message
print( 'Running ...' )
start_time = time.time()

#instantiate the component:
transport_component = SurfaceFlowTransport(mg, './stof_params.txt')

#perform the loop:
elapsed_time = 0. #total time in simulation
while elapsed_time < time_to_run:
    timestep = transport_component.set_and_return_dynamic_timestep()
    mg = transport_component.transport_sed(elapsed_time)
    elapsed_time += timestep

#Finalize and plot
zm = mg.at_node['planet_surface__elevation']
h = mg.at_node['planet_surface__water_depth']
ddz=zm-zinit
print ddz[np.where(ddz!=0.)]
print np.amax(ddz)
    
# Get a 2D array version of the water depths and elevations
hr = mg.node_vector_to_raster(h)
zr = mg.node_vector_to_raster(zm)
dzr = mg.node_vector_to_raster(ddz)

# Clear previous plots
pylab.figure(1)
pylab.close()
pylab.figure(2)
pylab.close()

# Plot topography
pylab.figure(1)
pylab.subplot(131)
im = pylab.imshow(zr, cmap=pylab.cm.RdBu)  # display a colored image
pylab.colorbar(im)
pylab.title('Topography')
    
# Plot change in topo
pylab.figure(1)
pylab.subplot(132)
im = pylab.imshow(dzr, cmap=pylab.cm.RdBu)  # display a colored image
pylab.colorbar(im)
pylab.title('Topo change')
    
# Plot water depth
pylab.subplot(133)
im2 = pylab.imshow(hr, cmap=pylab.cm.RdBu)  # display a colored image
#pylab.clim(0, 0.25)
pylab.colorbar(im2)
pylab.title('Water depth')
    
# Display the plots
pylab.show()
print('Done.')
print('Total run time = '+str(time.time()-start_time)+' seconds.')



#! /usr/bin/env python
"""
Unit tests for landlab.model_parameter_dictionary
"""

import unittest
import os
import tempfile
import numpy as np

from landlab import ModelParameterDictionary
from landlab.core.model_parameter_dictionary import (MissingKeyError,
                                                     ParameterValueError)

_TEST_PARAM_DICT_FILE = u"""
# A Comment
INT_VAL:
1

FLOAT_VAL: # A Comment
2.2
STRING_VAL:
The Landlab

TRUE_BOOL_VAL:
True
FALSE_BOOL_VAL:
False
"""

class TestModelParameterDictionary(unittest.TestCase):
    def setUp(self):
        from StringIO import StringIO
        self.param_file = StringIO(_TEST_PARAM_DICT_FILE)
        self.param_dict = ModelParameterDictionary()

        self.param_dict.read_from_file(self.param_file)

    def test_read_file(self):
        all_keys = set([
            'FLOAT_VAL', 'INT_VAL', 'STRING_VAL', 'TRUE_BOOL_VAL',
            'FALSE_BOOL_VAL',
        ])
        param_list = set(self.param_dict.params())

        self.assertEqual(param_list, all_keys)

    def test_read_file_name(self):
        (prm_fd, prm_file_name) = tempfile.mkstemp()

        prm_file = os.fdopen(prm_fd, 'w')
        prm_file.write(_TEST_PARAM_DICT_FILE)
        prm_file.close()

        param_dict = ModelParameterDictionary()
        param_dict.read_from_file(prm_file_name)

        os.remove(prm_file_name)

        all_keys = set([
            'FLOAT_VAL', 'INT_VAL', 'STRING_VAL', 'TRUE_BOOL_VAL',
            'FALSE_BOOL_VAL',
        ])
        param_list = set(param_dict.params())

        self.assertEqual(param_list, all_keys)


    def test_read_file_like_twice(self):
        from StringIO import StringIO
        param_file = StringIO(_TEST_PARAM_DICT_FILE)
        param_dict_1 = ModelParameterDictionary()
        param_dict_2 = ModelParameterDictionary()

        param_dict_1.read_from_file(param_file)
        param_dict_2.read_from_file(param_file)

    def test_read_int(self):
        self.assertEqual(self.param_dict.read_int('INT_VAL'), 1)

        with self.assertRaises(ParameterValueError):
            self.param_dict.read_int('FLOAT_VAL')

        with self.assertRaises(MissingKeyError):
            self.param_dict.read_int('MISSING_INT')

        self.assertEqual(self.param_dict.read_int('MISSING_INT', 2), 2)

    def test_get_int(self):
        self.assertEqual(self.param_dict.get('INT_VAL', ptype=int), 1)

        with self.assertRaises(ParameterValueError):
            self.param_dict.get('FLOAT_VAL', ptype=int)

        with self.assertRaises(MissingKeyError):
            self.param_dict.get('MISSING_INT', ptype=int)

    def test_set_default(self):
        self.param_dict.setdefault('MISSING_INT', 2)
        self.assertEqual(self.param_dict.read_int('MISSING_INT'), 2)

    def test_read_float(self):
        self.assertEqual(self.param_dict.read_float('FLOAT_VAL'), 2.2)
        self.assertEqual(self.param_dict.read_float('INT_VAL'), 1)

        with self.assertRaises(ParameterValueError):
            self.param_dict.read_float('STRING_VAL')

        with self.assertRaises(MissingKeyError):
            self.param_dict.read_float('MISSING_FLOAT')

    def test_read_string(self):
        self.assertEqual(self.param_dict.read_string('STRING_VAL'),
                         'The Landlab')
        self.assertEqual(self.param_dict.read_string('INT_VAL'), '1')
        self.assertEqual(self.param_dict.read_string('FLOAT_VAL'), '2.2')

        with self.assertRaises(MissingKeyError):
            self.param_dict.read_string('MISSING_STRING')

    def test_read_bool(self):
        self.assertEqual(self.param_dict.read_bool('TRUE_BOOL_VAL'), True)
        self.assertEqual(self.param_dict.read_bool('FALSE_BOOL_VAL'), False)

        with self.assertRaises(MissingKeyError):
            self.param_dict.read_bool('MISSING_BOOLEAN')

        with self.assertRaises(ParameterValueError):
            self.param_dict.read_bool('STRING_VAL')

    def test_dict_keys(self):
        all_keys = set([
            'FLOAT_VAL', 'INT_VAL', 'STRING_VAL', 'TRUE_BOOL_VAL',
            'FALSE_BOOL_VAL',
        ])
        self.assertEqual(set(self.param_dict), all_keys)
        for key in all_keys:
            self.assertTrue(key in self.param_dict)

    def test_dict_index(self):
        self.assertEqual(self.param_dict['INT_VAL'], '1')

class TestModelParameterDictionaryAutoType(unittest.TestCase):
    def setUp(self):
        from StringIO import StringIO

        _TEST_FILE = u"""
# A Comment
INT_VAL:
1
DBL_VAL:
1.2
STR_VAL:
    landlab
BOOL_VAL:
true
INT_ARRAY_VAL:
1,2 ,4 ,7

DBL_ARRAY_VAL:
1.,2. ,4. ,7.
        """
        self.param_dict = ModelParameterDictionary(
            auto_type=True, from_file=StringIO(_TEST_FILE))

    def test_auto_type(self):
        self.assertEqual(self.param_dict['INT_VAL'], 1)
        self.assertEqual(self.param_dict['DBL_VAL'], 1.2)
        self.assertEqual(self.param_dict['STR_VAL'], 'landlab')
        self.assertEqual(self.param_dict['BOOL_VAL'], True)

    def test_int_vector(self):
        val = self.param_dict['INT_ARRAY_VAL']

        self.assertEqual(list(val), [1, 2, 4, 7])
        self.assertIsInstance(val, np.ndarray)
        self.assertEqual(val.dtype, np.int)

    def test_float_vector(self):
        val = self.param_dict['DBL_ARRAY_VAL']

        self.assertEqual(list(val), [1., 2., 4., 7.])
        self.assertIsInstance(val, np.ndarray)
        self.assertEqual(val.dtype, np.float)


if __name__ == '__main__':
    unittest.main()



import numpy as np
from numpy.testing import assert_array_equal
from nose.tools import with_setup

from landlab import RasterModelGrid


def setup_unit_grid():
    globals().update({
        '_RMG': RasterModelGrid(4, 5)
    })


def setup_grid():
    globals().update({
        '_RMG': RasterModelGrid(4, 5, 12.)
    })


@with_setup(setup_unit_grid)
def test_unit_spacing():
    lengths = _RMG.calculate_link_length()
    assert_array_equal(lengths, np.ones(31))


@with_setup(setup_grid)
def test_non_unit_spacing():
    assert_array_equal(_RMG.calculate_link_length(),
                       _RMG.node_spacing * np.ones(31))


@with_setup(setup_grid)
def test_link_length():
    assert_array_equal(_RMG.link_length, 12 * np.ones(31))


@with_setup(setup_grid)
def test_active_link_length():
    assert_array_equal(_RMG.active_link_length, 12 * np.ones(17))



# -*- coding: utf-8 -*-
########## generate_overland_flow.py ###############
##
## This component simulates overland flow using
## the 2-D numerical model of shallow-water flow
## over topography using the Bates et al. (2010)
## algorithm for storage-cell inundation modeling.
##
## Most of code written by Greg Tucker.
## Shear stress component added by Nicole Gasparini
###########################################

from landlab import create_and_initialize_grid
from landlab import ModelParameterDictionary
import numpy as np
from matplotlib import pyplot as plt
import os


_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'input_data.txt')



class OverlandFlow(object):
    
    def __init__(self):
        
        ## I'M PRETTY SURE WE INITIALIZE EVERYTHING HERE AS NULL (BASED ON
        ## TYPE, AND WE ALSO INITIALIZE CONSTANTS --- IS h_init a constant?
        ## or do we want to emphasize that some initial depth (NOT ZERO)
        ## is required? For now - I'm making it a constant. - JMA
        
        self.grid = None
        self.input_file = None

        self.current_time = 0.0
        self.m_n = 0.0
        self.intensity = 0.0
        self.stormduration = 0.0
        self.m_n_sq = self.m_n*self.m_n # manning's n squared
       
        self.h_init = 0.001           ## initial depth
        self.g = 9.8               # gravitational acceleration (m/s2)
        self.alpha = 0.2           # time-step factor (ND; from Bates et al., 2010)
        self.rho = 1000 # density of water, kg/m^3  
        self.ten_thirds = 10./3.   # pre-calculate 10/3 for speed



    def initialize(self, grid=None, input_file=None, intensity=None, stormduration=None):
        
        self.grid = grid 
        if self.grid==None:
            self.grid = create_and_initialize_grid(input_file)
            
        ##self.current_time = current_time <- this isn't being used yet. 
        
        # Create a ModelParameterDictionary for the inputs
        
        MPD = ModelParameterDictionary()
        ## I am not sure we want to use an instance of MPD as an input_file. I'm confused about this. SO for 
        ## now I changed this to reflect other component types
        
        if input_file is None:
            input_file = _DEFAULT_INPUT_FILE
            
        MPD.read_from_file(input_file)
        
        ## This was the old way... where an instance of MPD is an input_file
        #if type(input_file)==MPD: 
        #    inputs = input_file
        #else:
        #    inputs = MPD(input_file)
        
        # Read input/configuration parameters
        self.m_n = MPD.read_float('MANNINGS_N')
        
        if intensity == None:
            self.rainfall_mmhr = MPD.read_float( 'RAINFALL_RATE')
        else:
            self.rainfall_mmhr = intensity
        
        if stormduration == None:
            self.rain_duration = MPD.read_float( 'RAIN_DURATION' )
        else:
            self.rain_duration = stormduration
          
        #self.h_init = 0.001        # initial thin layer of water (m)
        #self.g = 9.8               # gravitational acceleration (m/s2)
        #self.alpha = 0.2           # time-step factor (ND; from Bates et al., 2010)
        #self.m_n_sq = self.m_n*self.m_n # manning's n squared
        #self.rho = 1000 # density of water, kg/m^3     
        
        # Derived parameters ## THIS IS IMPORTANT - UNITS!!!! Double check precip component...
        self.rainfall_rate = (self.rainfall_mmhr/1000.)/3600.  # rainfall in m/s
        
        
        # Set up state variables
        
        self.hstart = grid.zeros(centering='node') + self.h_init 
        self.h = grid.zeros(centering='node') + self.h_init     # water depth (m)
        #NG why is water depth at each node and not at cells, which are only active
        self.q = grid.zeros(centering='active_link') # unit discharge (m2/s)
        self.dhdt = grid.zeros(centering='active_cell') # rate of water-depth change
        #NG why is dhdt at active cells, but h at nodes?
        #Maybe because you need a boundary condition of water depth at the 
        #boundary locations?  But water depths only change at active cells?
        
    def calculate_flow_at_one_point(self, grid, z, study_node, total_t=None, rainrate=None, rainduration=None):
        
        '''
        This method calculates discharge, water depth and shear stress at one
        point in the grid, defined as "study_node" in the function arguments.
        The study node is defined in the driver file, where the node ID must also
        be found using the grid_coords_to_node_id() function. 
        It is important to note that the study node does NOT have to be the outlet
        node, which must be defined to correctly account for boundary conditions.
        As it stands, using the outlet node as the "study_node" may cause significant
        boundary errors. Working on it!
        
        This function runs for the total run time, defined in the arguments as
        total_t (s). This function will work regardless of if rainduration (s) is shorter
        or longer than total_t (s)
        
        Rainrate is the rainfall intensity in m/s. 
        Rain duration is the total storm time in seconds.
        '''
    
        g=self.g
        alpha = self.alpha
        m_n_sq = self.m_n_sq
        ten_thirds = self.ten_thirds
        rho = self.rho
        
        if total_t==None:
            total_t = self.rain_duration
        if rainrate==None:
            rainrate = self.rainfall_rate
        if rainduration==None:
            rainduration = total_t
            
        #interior_nodes are the nodes on which you will be calculating flow 
        interior_nodes = grid.get_active_cell_node_ids()
        
        self.h = self.hstart
        h = self.h
        q = self.q
        dhdt = self.dhdt
        
        elapsed_time = 0
        
         # Get a list of the interior cells
        interior_cells = grid.get_active_cell_node_ids()
    
        # To track discharge at the outlet through time, we create initially empty
        # lists for time and outlet discharge.
        
        ## outlet_node AKA "study_node" = (input)
        q_outlet = []
        t = []
        t_1 = []
        h_1 = []
        q_outlet.append(0.)
        t.append(0.)
        t_1.append(0.)
        h_1.append(0.)
        
        nbr_node = grid.find_node_in_direction_of_max_slope_d4(z, study_node)
        study_link = grid.get_active_link_connecting_node_pair(study_node, nbr_node)


        # Main loop
        while elapsed_time < total_t:
        
        
            # Calculate time-step size for this iteration (Bates et al., eq 14)
            dtmax = alpha*grid.dx/np.sqrt(g*np.amax(h))
            #print "dtmax", dtmax
            
            if elapsed_time+dtmax > total_t:
                dtmax = total_t - elapsed_time
        
            # Calculate the effective flow depth at active links. Bates et al. 2010
            # recommend using the difference between the highest water-surface
            # and the highest bed elevation between each pair of cells.
            zmax = grid.max_of_link_end_node_values(z)
            w = h+z   # water-surface height
            wmax = grid.max_of_link_end_node_values(w)
            hflow = wmax - zmax
        
            # Calculate water-surface slopes
            water_surface_slope = grid.calculate_gradients_at_active_links(w)
       
            # Calculate the unit discharges (Bates et al., eq 11)
            q = (q-g*hflow*dtmax*water_surface_slope)/ \
                (1.+g*hflow*dtmax*0.06*0.06*abs(q)/(hflow**ten_thirds))
            
            #print "q study link", q[outlet_link] 
        
            # Calculate water-flux divergence at nodes
            dqds = grid.calculate_flux_divergence_at_nodes(q)
        
            # Update rainfall rate
            if elapsed_time > rainduration:
                rainrate = 0.
        
            # Calculate rate of change of water depth
            dhdt = rainrate-dqds
        
            # Second time-step limiter (experimental): make sure you don't allow
            # water-depth to go negative
            if np.amin(dhdt) < 0.:
                shallowing_locations = np.where(dhdt<0.)
                time_to_drain = -h[shallowing_locations]/dhdt[shallowing_locations]
                dtmax2 = alpha*np.amin(time_to_drain)
                dt = np.min([dtmax, dtmax2])
            else:
                dt = dtmax
        
            # Update the water-depth field
            h[interior_cells] = h[interior_cells] + dhdt[interior_cells]*dt

            w_slp_studypoint,garbage=grid.calculate_max_gradient_across_node_d4(w,study_node)
            tau_temp=self.rho*self.g*w_slp_studypoint*self.h[study_node]
            t_1.append(tau_temp)

            # Update model run time
            elapsed_time += dt
            print "elapsed time", elapsed_time
        
            # Remember discharge and time
            t.append(elapsed_time)
            q_outlet.append(q[study_link])            
            h_1.append(h[study_node])


        
        plt.figure('Discharge at Study Node')
        plt.plot(t, q_outlet, 'r-')
        plt.legend(loc=1)
        plt.ylabel('Discharge, m^3/s')
        plt.xlabel('Time, s')
        
        plt.figure('Shear Stress at Study Node')
        plt.plot(t, t_1, 'r--')
        plt.ylabel('Shear Stress, Pa')
        plt.xlabel('Time, s')
        plt.legend(loc=1)
        
        plt.figure('Water Depth at Study Node')
        plt.plot(t, h_1, 'r--')
        plt.ylabel('Water Depth, m')
        plt.xlabel('Time, s')
        plt.legend(loc=1)

     #   
        plt.show()
        #pylab.show()
        

    #def generate_overland_flow_across_grid()    
    #def run_one_step_internal(self, delt):
    #    
    #    # Take the smaller of delt or built-in time-step size self.dt
    #    dt = min(self.dt, delt)
    #    
    #    # Calculate the gradients and sediment fluxes
    #    self.g = self.grid.calculate_gradients_at_active_links(self.z)
    #    self.qs = -self.kd*self.g
    #    
    #    # Calculate the net deposition/erosion rate at each node
    #    self.dqsds = self.grid.calculate_flux_divergence_at_nodes(self.qs)
    #    
    #    # Calculate the total rate of elevation change
    #    dzdt = self.uplift_rate - self.dqsds
    #    
    #    # Update the elevations
    #    self.z[self.interior_cells] += dzdt[self.interior_cells] * dt
    #                             
    #    # Update current time and return it
    #    self.current_time += dt
    #    return self.current_time
    #    
    #    
    #def run_until_explicit(self, grid, z, t):
    #    
    #    while self.current_time < t:
    #        remaining_time = t - self.current_time
    #        z, g, qs, dqsds, dzdt = self.run_one_step_explicit(mg, z, g, qs, dqsds, dzdt, remaining_time)
    #        
    #    return z, g, qs, dqsds, dzdt
    #    
    #    
    #def run_until_internal(self, t):
    #    
    #    while self.current_time < t:
    #        remaining_time = t - self.current_time
    #        self.run_one_step_internal(remaining_time)
    #                
    #    
    #def get_time_step(self):
    #    """
    #    Returns time-step size.
    #    """
    #    return self.dt



#! /usr/bin/env python

import numpy as np

from landlab.components.flexure import FlexureComponent
from landlab import RasterModelGrid
from landlab.plot import imshow_field


SHAPE = (100, 100)
SPACING = (10e3, 10e3)
LOAD_LOCS = [
    (SHAPE[0] / 2, SHAPE[1] / 2),
    (SHAPE[0] / 4, SHAPE[1] * 3 / 4),
]


def put_two_point_loads_on_grid(grid):
    load = grid.field_values('node', 'lithosphere__overlying_pressure')
    load = load.view()
    load.shape = grid.shape
    for loc in LOAD_LOCS:
        load[loc] = 1e15


def create_lithosphere_elevation_with_bulge(grid):
    grid.add_zeros('node', 'lithosphere__elevation')

    z = grid.field_values('node', 'lithosphere__elevation').view()
    z.shape = grid.shape

    (y, x) = np.meshgrid(np.linspace(0, np.pi * .5, grid.shape[0]),
                         np.linspace(0, np.pi * .5, grid.shape[1]))
    (x0, y0) = (np.pi / 3, np.pi / 8)
    np.sin((x - x0) ** 2 + (y - y0) ** 2, out=z)


def main():

    grid = RasterModelGrid(SHAPE[0], SHAPE[1], SPACING[0])

    create_lithosphere_elevation_with_bulge(grid)

    flex = FlexureComponent(grid, method='flexure')

    put_two_point_loads_on_grid(grid)

    flex.update()

    grid.imshow('node', 'lithosphere__elevation', symmetric_cbar=False,
                show=True) 


if __name__ == '__main__':
    main()



#! /usr/bin/env python

import numpy as np

from landlab.components.flexure import FlexureComponent
from landlab import RasterModelGrid


def get_random_load_locations(shape, n_loads):
    return np.random.random_integers(0, shape[0] * shape[1] - 1, n_loads)


def get_random_load_magnitudes(n_loads):
    return np.random.normal(1e9, 1e12, n_loads)


def put_loads_on_grid(grid, load_locations, load_sizes):
    load = grid.at_node['lithosphere__overlying_pressure'].view()
    for (loc, size) in zip(load_locations, load_sizes):
        load.flat[loc] = size


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--n-loads', type=int, default=16,
                        help='Number of loads to apply')
    parser.add_argument('--shape', type=int, default=200,
                        help='Number rows and columns')
    parser.add_argument('--spacing', type=int, default=5e3,
                        help='Spading between rows and columns (m)')
    parser.add_argument('--n-procs', type=int, default=1,
                        help='Number of processors to use')
    parser.add_argument('--plot', action='store_true', default=False,
                        help='Plot an image of the total deflection')

    args = parser.parse_args()

    shape = (args.shape, args.shape)
    spacing = (args.spacing, args.spacing)

    load_locs = get_random_load_locations(shape, args.n_loads)
    load_sizes = get_random_load_magnitudes(args.n_loads)

    grid = RasterModelGrid(shape[0], shape[1], spacing[0])

    flex = FlexureComponent(grid, method='flexure')

    put_loads_on_grid(grid, load_locs, load_sizes)

    flex.update(n_procs=args.n_procs)

    grid.imshow('node', 'lithosphere__elevation', symmetric_cbar=False,
                cmap='spectral', show=True) 


if __name__ == '__main__':
    main()



########## ET_PriestlyTaylor.py ###############
##
##  This component calculates and updates Priestly Taylor Evapotranspiration.
##
##  Equations from ACSE EWRI report (Jan 2005) are used
##  Inputs are read from the file 'PET_input.txt'
##  
##  
##
## Written by Sai Nudurupati & E.I. Nov 2013.
###########################################

import os
from numpy import * 
from landlab import ModelParameterDictionary
from math import *
from matplotlib.pyplot import *

""" Default input file """
_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'PET_input.txt')


class ET():

    def __init__( self ):
        pass

    def initialize( self ):

        """ Reading data from input file """
        MPD = ModelParameterDictionary()
        MPD.read_from_file( _DEFAULT_INPUT_FILE )

        """ Priestly Taylor Constant """
        self._alpha = MPD.read_float( 'alpha' )
        """ Surface Albedo """
        self._a = MPD.read_float( 'a' )
        """ Latent Heat of Vaporization """
        self._pwhv = MPD.read_float( 'pwhv' )
        """ Psychometric Constant """
        self._y = MPD.read_float( 'y' )
        """ Stefan Boltzmann Constant """
        self._sigma = MPD.read_float( 'sigma' )
        """ Solar Constant """
        self._Gsc = MPD.read_float( 'Gsc' )
        """ Latitude of Location under Study """
        self._lat = MPD.read_float( 'lat' )
        """ Elevation of Location under Study """
        self._z = MPD.read_float( 'z' )
        """ Adjustment Coefficient """
        self._Krs = MPD.read_float( 'Krs' )        

        """ Initialize Arrays """

        """ Net Short Wave Radiation """
        self._Rns = 0
        """ Net Radiation """
        self._Rn = 0
        """ Clear Sky Solar Radiation """
        self._Rso = 0
        """ Short Wave Radiation """
        self._Rs = 0
        """ Relative cloudiness factor """
        self._u = 0
        """ Priestly Taylor evapotranspiration """
        self._ETp = 0
        """ Extraterrestrial Radiation """
        self._Ra = 0
        """ Julian Day """
        self._J = 0
        """ Sunset hour angle """
        self._ws = 0
        self._x = 0
        """ Solar declination angle """
        self._sdecl = 0
        """ Inverse relative distance factor """
        self._dr = 0
        """ Cloudiness function """
        self._fcd = 0
        """ Latitude in radians """
        self._phi = (3.14/180)*self._lat

           
    def update( self, TIME ):

        """
            Julian Day - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (25)
        """        
        self._J = floor( (TIME - floor( TIME)) * 365 ) 

        """
            Solar Declination Angle - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (51)
        """
        self._sdecl = 0.409*sin((((2.0*3.14)/365.0)*self._J)-1.39)

        """
            Inverse Relative Distance Factor - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (50)
        """
        self._dr = 1 + (0.033*cos((2.0*3.14/365.0)*self._J))

        """
            To calculate ws - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (61)
        """
        self._x = 1.0-((pow((tan(self._phi)),2.0))*(pow(tan(self._sdecl),2.0)))  
        if self._x <= 0:
            self._x = 0.00001;
            
        """
            Sunset Hour Angle - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (60)
        """
        self._ws = (3.14/2.0)-atan((-1*tan(self._phi)*tan(self._sdecl))/(pow(self._x,2.0)))

        """
            Extraterrestrial radmodel.docx - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (48)
        """
        self._Ra = 11.57*(24.0/3.14)*4.92*self._dr*((self._ws*sin(self._phi)*sin(self._sdecl))+ \
                        (cos(self._phi)*cos(self._sdecl)*(sin(self._ws))))

        """
            Clear-sky Solar Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (47)
        """
        self._Rs = (0.75+((2.0*pow(10,-5.0))*self._z))*self._Ra

        """
            Net Short Wave Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (43)
        """
        self._Rns = self._Rs*(1-self._a)

        """
            Relative Cloudiness - ASCE-EWRI Task Committee Report, Jan-2005 - Page 35
        """
        self._u = 0.3
        
        """
            Cloudiness Function - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (45)
        """
        self._fcd = (1.35*self._u)-0.35

        """
            Net Radiation - ASCE-EWRI Task Committee Report, Jan-2005 - Eqn (42)
        """                                         
        self._Rn = self._Rns 

        """
            Priestly Taylor Evapotranspiration in mm/day - Priestly et. al. 1972
        """
        self._ETp = max(self._alpha*(self._delta/(self._delta+self._y) \
                        )*(self._Rn/self._pwhv), 0)                                                                                                       


    def plott( self ):

        figure(1)
        self._px = xrange( 1, self._length+1)
        plot( self._px, self._Rs, label = 'Modeled' )
        hold(True)
        plot( self._px, self._RsObs, '*', label = 'Observed' )
        hold(False)
        xlabel( 'Days' )
        ylabel( 'Short Wave Radiation' )
        title( 'Shortwave Radiation' )
        legend()
        

        figure(2)
        plot( self._px, self._ETp )
        xlabel( 'Days' )
        ylabel( 'Evapotranspiration PET (in mm/day)' )
        title( 'Evapotranspiration' )
        show()




from landlab.components.craters.dig_craters import impactor
from landlab import ModelParameterDictionary

from landlab import RasterModelGrid
import numpy as np
import time
import pylab

#get the needed properties to build the grid:
input_file = './craters_params.txt'
inputs = ModelParameterDictionary(input_file)
nrows = inputs.read_int('nrows')
ncols = inputs.read_int('ncols')
dx = inputs.read_float('dx')
leftmost_elev = inputs.read_float('leftmost_elevation')
initial_slope = inputs.read_float('initial_slope')
nt = inputs.read_int('number_of_craters_per_loop')
loops = inputs.read_int('number_of_loops')

mg = RasterModelGrid(nrows, ncols, dx)
mg.set_inactive_boundaries(False, False, False, False)

#create the fields in the grid
mg.create_node_array_zeros('planet_surface__elevation')
mg['node'][ 'planet_surface__elevation'] = np.load('init.npy')

# Display a message
print( 'Running ...' )
start_time = time.time()

#instantiate the component:
craters_component = impactor(mg, input_file)
craters_component.radius_auto_flag = 0
craters_component.position_auto_flag = 0
craters_component.angle_auto_flag = 0

#perform the loops:
slope = np.empty(nt)
mass_balance = np.empty(nt)
redug_crater = np.empty(nt)
for i in xrange(loops):
    x = np.load('x_'+str((i+1)*nt)+'.npy')
    y = np.load('y_'+str((i+1)*nt)+'.npy')
    r = np.load('r_'+str((i+1)*nt)+'.npy')
    angle = np.load('angle_'+str((i+1)*nt)+'.npy')
    az = np.load('az_'+str((i+1)*nt)+'.npy')
    for j in xrange(nt):
        craters_component._xcoord = x[j]
        craters_component._ycoord = y[j]
        craters_component._radius = r[j]
        craters_component._angle_to_vertical = angle[j]
        craters_component._azimuth_of_travel = az[j]
        mg = craters_component.excavate_a_crater_noangle(mg)
        slope[j] = craters_component.impact_property_dict['surface_slope']
        mass_balance[j] = craters_component.impact_property_dict['mass_balance']
        redug_crater[j] = craters_component.impact_property_dict['redug_crater']
        print 'Completed loop ', j
    mystring = 'craterssave'+str((i+1)*nt)
    np.save(mystring,mg['node']['planet_surface__elevation'])
    #Save the properties
    np.save(('slope_'+str((i+1)*nt)),slope)
    np.save(('mass_balance_'+str((i+1)*nt)),mass_balance)
    np.save(('redug_crater_'+str((i+1)*nt)),redug_crater)

#Finalize and plot
elev = mg['node']['planet_surface__elevation']
elev_r = mg.node_vector_to_raster(elev)
# Clear previous plots
#pylab.figure(1)
#pylab.close()
#pylab.figure(1)
#im = pylab.imshow(elev_r, cmap=pylab.cm.RdBu)  # display a colored image
#pylab.colorbar(im)
#pylab.title('Topography')

print('Done.')
print('Total run time = '+str(time.time()-start_time)+' seconds.')

#pylab.show()



#! /usr/bin/env python
"""
Unit tests for landlab.io.netcdf module.
"""

import unittest
import os
import numpy as np
from StringIO import StringIO

from landlab import RasterModelGrid
from landlab.io.netcdf import write_netcdf, NotRasterGridError
from landlab.io.netcdf.read import _get_raster_spacing


_TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), 'data')


class TestWriteNetcdf(unittest.TestCase):
    def test_write(self):
        field = RasterModelGrid(4, 3)
        #field.new_field_location('node', 12.)
        values = np.arange(12.)
        field.add_field('node', 'planet_surface__elevation', values)

        write_netcdf('test.nc', field)

        import netCDF4 as nc

        root = nc.Dataset('test.nc', 'r', format='NETCDF4')
        self.assertEqual(set(root.dimensions), set(['ni', 'nj', 'nt']))
        self.assertEqual(len(root.dimensions['ni']), 3)
        self.assertEqual(len(root.dimensions['nj']), 4)
        self.assertTrue(len(root.dimensions['nt']), 1)
        self.assertTrue(root.dimensions['nt'].isunlimited())

        self.assertEqual(set(root.variables),
                         set(['x', 'y', 'planet_surface__elevation']))

        self.assertListEqual(list(root.variables['x'][:].flat),
                             [0., 1., 2.,
                              0., 1., 2.,
                              0., 1., 2.,
                              0., 1., 2., ])
        self.assertListEqual(list(root.variables['y'][:].flat),
                             [0., 0., 0.,
                              1., 1., 1.,
                              2., 2., 2.,
                              3., 3., 3., ])
        self.assertListEqual(
            list(root.variables['planet_surface__elevation'][:].flat),
            range(12))
        root.close()


class TestGetRasterSpacing2D(unittest.TestCase):
    def test_unit_spacing(self):
        (x, y) = np.meshgrid(np.arange(5.), np.arange(4.))

        spacing = _get_raster_spacing((y, x))
        self.assertEqual(spacing, 1.)

    def test_non_unit_spacing(self):
        (x, y) = np.meshgrid(np.arange(5.) * 2, np.arange(4.) * 2)

        spacing = _get_raster_spacing((y, x))
        self.assertEqual(spacing, 2.)

    def test_uneven_spacing_axis_0(self):
        (x, y) = np.meshgrid(np.logspace(0., 2., num=5), np.arange(4.))

        with self.assertRaises(NotRasterGridError):
            spacing = _get_raster_spacing((y, x))

    def test_uneven_spacing_axis_1(self):
        (x, y) = np.meshgrid(np.arange(4.), np.logspace(0., 2., num=5))

        with self.assertRaises(NotRasterGridError):
            spacing = _get_raster_spacing((y, x))

    def test_switched_coords(self):
        (x, y) = np.meshgrid(np.arange(5.), np.arange(4.))

        spacing = _get_raster_spacing((x, y))
        self.assertEqual(spacing, 0.)


class TestGetRasterSpacing1D(unittest.TestCase):
    def test_unit_spacing(self):
        spacing = _get_raster_spacing((np.arange(5.), ))
        self.assertEqual(spacing, 1.)

    def test_non_unit_spacing(self):
        spacing = _get_raster_spacing((np.arange(5.) * 2, ))
        self.assertEqual(spacing, 2.)

    def test_uneven_spacing(self):
        (x, y) = np.meshgrid(np.logspace(0., 2., num=5), np.arange(4.))

        with self.assertRaises(NotRasterGridError):
            spacing = _get_raster_spacing((np.logspace(0., 2., num=5), ))


if __name__ == '__main__':
    unittest.main()
